==> qemu_9.2.1+ds.orig.tar.xz: qemu-9.2.1/tests/lcitool/libvirt-ci/lcitool/containers/podman.py <==
# podman.py - module implementing podman container runtime wrapper logic
#
# Copyright (c) 2023 Abdulwasiu Apalowo.
#
# SPDX-License-Identifier: GPL-2.0-or-later

import json
import logging

from .containers import Container

log = logging.getLogger()


class Podman(Container):
    """Podman container class"""

    def run(self, image, container_cmd, user, tempdir, env=None, datadir=None,
            script=None, **kwargs):
        """
        Prepares and runs the command inside a container.

        See Container.run() for more information.
        """

        return super().run(image, container_cmd, user, tempdir, env, datadir,
                           script, **kwargs)

    def shell(self, image, user, tempdir, env=None, datadir=None, script=None,
              **kwargs):
        """
        Spawns an interactive shell inside the container.

        See Container.shell() for more information
        """

        return super().shell(image, user, tempdir, env, datadir,
                             script, **kwargs)

    def _extra_args(self, user):
        """
        Get Podman specific host namespace mapping
        :param user: numerical ID (int) or username (str) of the user.

        :returns: a list of id mapping
        """

        # Podman cannot reuse host namespace when running non-root
        # containers.  Until support for --keep-uid is added we can
        # just create another mapping that will do that for us.
        # Beware, that in {uid,gid}map=container_id:host_id:range, the
        # host_id does actually refer to the uid in the first mapping
        # where 0 (root) is mapped to the current user and rest is
        # offset.
        #
        # In order to set up this mapping, we need to keep all the
        # user IDs to prevent possible errors as some images might
        # expect UIDs up to 90000 (looking at you fedora), so we don't
        # want the overflowuid to be used for them.  For mapping all
        # the other users properly, some math needs to be done.
        # Don't worry, it's just addition and subtraction.
        #
        # 65536 ought to be enough (tm), but for really rare cases the
        # maximums might need to be higher, but that only happens when
        # your /etc/sub{u,g}id allow users to have more IDs.  Unless
        # --keep-uid is supported, let's do this in a way that should
        # work for everyone.

        podman_args_ = []

        _, _, uid, gid, _, _, _ = self._passwd(user)
        if uid == 0:
            return podman_args_

        max_uid = int(open("/etc/subuid").read().split(":")[-1])
        max_gid = int(open("/etc/subgid").read().split(":")[-1])

        if max_uid is None:
            max_uid = 65536
        if max_gid is None:
            max_gid = 65536

        uid_other = uid + 1
        gid_other = gid + 1
        uid_other_range = max_uid - uid
        gid_other_range = max_gid - gid

        podman_args_.extend([
            ("--uidmap", f"0:1:{uid}"),
            ("--uidmap", f"{uid}:0:1"),
            ("--uidmap", f"{uid_other}:{uid_other}:{uid_other_range}"),
            ("--gidmap", f"0:1:{gid}"),
            ("--gidmap", f"{gid}:0:1"),
            ("--gidmap", f"{gid_other}:{gid_other}:{gid_other_range}"),
        ])
        return podman_args_

    def _build_args(self, user, tempdir, env=None, datadir=None, script=None):
        """
        Options for Podman engine.

        :returns: a list containing id mapping. e.g
        [
            '--user', '10:10',
            '--volume', '/home/path:/container/path',
            '--uidmap', '0:1:1000',
            '--gidmap', '0:1:1000'
        ]
        """

        args_ = super()._build_args(
            user, tempdir, env=env, datadir=datadir, script=script
        )
        args_podman = args_ + self._extra_args(user)
        log.debug(f"Options for podman engine: {args_podman}")

        return args_podman

    def _images(self):
        """
        Get all container images.

        :returns: a list of image details
        """

        img = super()._images()
        images = json.loads(img)

        log.debug(f"Deserialized {self.engine} images\n%s", images)
        return images

    def image_exists(self, image_ref, image_tag):
        """
        Check if image exists in podman.
        :param image_ref: name/id/registry-path of image to check (str).
        :param image_tag: image tag (str).

        :returns: boolean
        """

        image_repository, _, image_name = image_ref.rpartition('/')

        for img in self._images():
            id = img.get("Id")
            img_repository = img.get("Names")

            image_reference = image_name + ":" + image_tag
            if image_repository:
                image_reference = image_repository + '/' + image_reference
                repository_names = img_repository
            else:
                # parse `img_repository` just to get "<image_name>:<image_tag>"
                repository_names = list(map(lambda x: x.split('/')[-1], img_repository))

            if (image_ref and id.startswith(image_ref)) or (image_reference in repository_names):
                return True

        return False

==> qemu_10.2.0+ds.orig.tar.xz: qemu-10.2.0/tests/lcitool/libvirt-ci/lcitool/containers/podman.py <==
# podman.py - module implementing podman container runtime wrapper logic
#
# Copyright (c) 2023 Abdulwasiu Apalowo.
#
# SPDX-License-Identifier: GPL-2.0-or-later

import json
import logging

from .containers import Container
from typing import Any, List, Optional, Tuple, Union
from pathlib import Path

log = logging.getLogger()


class Podman(Container):
    """Podman container class"""

    def run(
        self,
        image: str,
        container_cmd: str,
        user: Union[int, str],
        tempdir: Path,
        env: Optional[List[str]] = None,
        datadir: Optional[Path] = None,
        script: Optional[Path] = None,
        **kwargs: Any,
    ) -> int:
        """
        Prepares and runs the command inside a container.

        See Container.run() for more information.
        """

        return super().run(
            image, container_cmd, user, tempdir, env, datadir, script, **kwargs
        )

    def shell(
        self,
        image: str,
        user: Union[int, str],
        tempdir: Path,
        env: Optional[List[str]] = None,
        datadir: Optional[Path] = None,
        script: Optional[Path] = None,
        **kwargs: Any,
    ) -> int:
        """
        Spawns an interactive shell inside the container.

        See Container.shell() for more information
        """

        return super().shell(image, user, tempdir, env, datadir, script, **kwargs)

    def _extra_args(self, user: Optional[Union[int, str]]) -> List[Tuple[str, str]]:
        """
        Get Podman specific host namespace mapping
        :param user: numerical ID (int) or username (str) of the user.

        :returns: a list of id mapping
        """

        # Podman cannot reuse host namespace when running non-root
        # containers.  Until support for --keep-uid is added we can
        # just create another mapping that will do that for us.
        # Beware, that in {uid,gid}map=container_id:host_id:range, the
        # host_id does actually refer to the uid in the first mapping
        # where 0 (root) is mapped to the current user and rest is
        # offset.
        #
        # In order to set up this mapping, we need to keep all the
        # user IDs to prevent possible errors as some images might
        # expect UIDs up to 90000 (looking at you fedora), so we don't
        # want the overflowuid to be used for them.  For mapping all
        # the other users properly, some math needs to be done.
        # Don't worry, it's just addition and subtraction.
        #
        # 65536 ought to be enough (tm), but for really rare cases the
        # maximums might need to be higher, but that only happens when
        # your /etc/sub{u,g}id allow users to have more IDs.  Unless
        # --keep-uid is supported, let's do this in a way that should
        # work for everyone.

        podman_args_: List[Tuple[str, str]] = []

        _, _, uid, gid, _, _, _ = self._passwd(user)
        if uid == 0:
            return podman_args_

        max_uid = int(open("/etc/subuid").read().split(":")[-1])
        max_gid = int(open("/etc/subgid").read().split(":")[-1])

        if max_uid is None:
            max_uid = 65536
        if max_gid is None:
            max_gid = 65536

        uid_other = uid + 1
        gid_other = gid + 1
        uid_other_range = max_uid - uid
        gid_other_range = max_gid - gid

        podman_args_.extend(
            [
                ("--uidmap", f"0:1:{uid}"),
                ("--uidmap", f"{uid}:0:1"),
                ("--uidmap", f"{uid_other}:{uid_other}:{uid_other_range}"),
                ("--gidmap", f"0:1:{gid}"),
                ("--gidmap", f"{gid}:0:1"),
                ("--gidmap", f"{gid_other}:{gid_other}:{gid_other_range}"),
            ]
        )
        return podman_args_

    def _build_args(
        self,
        user: Union[int, str],
        tempdir: Path,
        env: Optional[List[str]] = None,
        datadir: Optional[Path] = None,
        script: Optional[Path] = None,
    ) -> List[Union[Tuple[str, str], Tuple[str]]]:
        """
        Options for Podman engine.

        :returns: a list containing id mapping. e.g
        [
            '--user', '10:10',
            '--volume', '/home/path:/container/path',
            '--uidmap', '0:1:1000',
            '--gidmap', '0:1:1000'
        ]
        """

        args_ = super()._build_args(
            user, tempdir, env=env, datadir=datadir, script=script
        )
        args_podman = args_ + self._extra_args(user)
        log.debug(f"Options for podman engine: {args_podman}")

        return args_podman

    def _images(self) -> Any:
        """
        Get all container images.

        :returns: a list of image details
        """

        img = super()._images()
        images = json.loads(img)

        log.debug(f"Deserialized {self.engine} images\n%s", images)
        return images

    def image_exists(self, image_ref: str, image_tag: str) -> bool:
        """
        Check if image exists in podman.
        :param image_ref: name/id/registry-path of image to check (str).
        :param image_tag: image tag (str).

        :returns: boolean
        """

        image_repository, _, image_name = image_ref.rpartition("/")

        for img in self._images():
            id = img.get("Id")
            img_repository = img.get("Names")

            image_reference = image_name + ":" + image_tag
            if image_repository:
                image_reference = image_repository + "/" + image_reference
                repository_names = img_repository
            else:
                # parse `img_repository` just to get "<image_name>:<image_tag>"
                repository_names = list(map(lambda x: x.split("/")[-1], img_repository))

            if (image_ref and id.startswith(image_ref)) or (
                image_reference in repository_names
            ):
                return True

        return False
